from typing import Final

from .solders import (
    COMPUTE_BUDGET_ID as _ID,
)
from .solders import (
    ComputeBudget,
    Pubkey,
    request_heap_frame,
    set_compute_unit_limit,
    set_compute_unit_price,
)

ID: Final[Pubkey] = _ID
"""Compute buddget program ID"""

__all__ = [
    "ComputeBudget",
    "request_heap_frame",
    "set_compute_unit_limit",
    "set_compute_unit_price",
    "ID",
]



from typing import Union

from ..solders import (
    BlockCleanedUp,
    BlockCleanedUpMessage,
    BlockNotAvailable,
    BlockNotAvailableMessage,
    BlockStatusNotAvailableYet,
    BlockStatusNotAvailableYetMessage,
    InternalErrorMessage,
    InvalidParamsMessage,
    InvalidRequestMessage,
    KeyExcludedFromSecondaryIndex,
    KeyExcludedFromSecondaryIndexMessage,
    LongTermStorageSlotSkipped,
    LongTermStorageSlotSkippedMessage,
    MethodNotFoundMessage,
    MinContextSlotNotReached,
    MinContextSlotNotReachedMessage,
    NodeUnhealthy,
    NodeUnhealthyMessage,
    ParseErrorMessage,
    RpcCustomErrorFieldless,
    ScanError,
    ScanErrorMessage,
    SendTransactionPreflightFailure,
    SendTransactionPreflightFailureMessage,
    SlotSkipped,
    SlotSkippedMessage,
    TransactionPrecompileVerificationFailure,
    TransactionPrecompileVerificationFailureMessage,
    UnsupportedTransactionVersion,
    UnsupportedTransactionVersionMessage,
)

RpcCustomError = Union[
    RpcCustomErrorFieldless,
    BlockCleanedUp,
    SendTransactionPreflightFailure,
    BlockNotAvailable,
    NodeUnhealthy,
    TransactionPrecompileVerificationFailure,
    SlotSkipped,
    LongTermStorageSlotSkipped,
    BlockCleanedUp,
    KeyExcludedFromSecondaryIndex,
    ScanError,
    BlockStatusNotAvailableYet,
    MinContextSlotNotReached,
    UnsupportedTransactionVersion,
]

__all__ = [
    "RpcCustomError",
    "BlockCleanedUp",
    "BlockCleanedUpMessage",
    "SendTransactionPreflightFailure",
    "SendTransactionPreflightFailureMessage",
    "RpcCustomErrorFieldless",
    "BlockNotAvailable",
    "BlockNotAvailableMessage",
    "NodeUnhealthy",
    "NodeUnhealthyMessage",
    "TransactionPrecompileVerificationFailure",
    "TransactionPrecompileVerificationFailureMessage",
    "SlotSkipped",
    "SlotSkippedMessage",
    "LongTermStorageSlotSkipped",
    "LongTermStorageSlotSkippedMessage",
    "KeyExcludedFromSecondaryIndex",
    "KeyExcludedFromSecondaryIndexMessage",
    "ScanError",
    "ScanErrorMessage",
    "BlockStatusNotAvailableYet",
    "BlockStatusNotAvailableYetMessage",
    "MinContextSlotNotReached",
    "MinContextSlotNotReachedMessage",
    "UnsupportedTransactionVersion",
    "UnsupportedTransactionVersionMessage",
    "ParseErrorMessage",
    "InvalidRequestMessage",
    "MethodNotFoundMessage",
    "InvalidParamsMessage",
    "InternalErrorMessage",
]



from typing import Any, List, Sequence, TypeVar, Union

from ..solders import (
    AccountNotification,
    AccountNotificationJsonParsed,
    AccountNotificationJsonParsedResult,
    AccountNotificationResult,
    BlockNotification,
    BlockNotificationResult,
    BlockStoreError,
    GetAccountInfoJsonParsedResp,
    GetAccountInfoMaybeJsonParsedResp,
    GetAccountInfoResp,
    GetBalanceResp,
    GetBlockCommitmentResp,
    GetBlockHeightResp,
    GetBlockProductionResp,
    GetBlockResp,
    GetBlocksResp,
    GetBlocksWithLimitResp,
    GetBlockTimeResp,
    GetClusterNodesResp,
    GetEpochInfoResp,
    GetEpochScheduleResp,
    GetFeeForMessageResp,
    GetFirstAvailableBlockResp,
    GetGenesisHashResp,
    GetHealthResp,
    GetHighestSnapshotSlotResp,
    GetIdentityResp,
    GetInflationGovernorResp,
    GetInflationRateResp,
    GetInflationRewardResp,
    GetLargestAccountsResp,
    GetLatestBlockhashResp,
    GetLeaderScheduleResp,
    GetMaxRetransmitSlotResp,
    GetMaxShredInsertSlotResp,
    GetMinimumBalanceForRentExemptionResp,
    GetMultipleAccountsJsonParsedResp,
    GetMultipleAccountsMaybeJsonParsedResp,
    GetMultipleAccountsResp,
    GetProgramAccountsJsonParsedResp,
    GetProgramAccountsMaybeJsonParsedResp,
    GetProgramAccountsResp,
    GetProgramAccountsWithContextJsonParsedResp,
    GetProgramAccountsWithContextMaybeJsonParsedResp,
    GetProgramAccountsWithContextResp,
    GetRecentPerformanceSamplesResp,
    GetSignaturesForAddressResp,
    GetSignatureStatusesResp,
    GetSlotLeaderResp,
    GetSlotLeadersResp,
    GetSlotResp,
    GetSupplyResp,
    GetTokenAccountBalanceResp,
    GetTokenAccountsByDelegateJsonParsedResp,
    GetTokenAccountsByDelegateResp,
    GetTokenAccountsByOwnerJsonParsedResp,
    GetTokenAccountsByOwnerResp,
    GetTokenLargestAccountsResp,
    GetTokenSupplyResp,
    GetTransactionCountResp,
    GetTransactionResp,
    GetVersionResp,
    GetVoteAccountsResp,
    IsBlockhashValidResp,
    LogsNotification,
    LogsNotificationResult,
    MinimumLedgerSlotResp,
    ProgramNotification,
    ProgramNotificationJsonParsed,
    ProgramNotificationJsonParsedResult,
    ProgramNotificationResult,
    RequestAirdropResp,
    RootNotification,
    RpcAccountBalance,
    RpcBlockCommitment,
    RpcBlockhash,
    RpcBlockProduction,
    RpcBlockProductionRange,
    RpcBlockUpdate,
    RpcConfirmedTransactionStatusWithSignature,
    RpcContactInfo,
    RpcIdentity,
    RpcInflationGovernor,
    RpcInflationRate,
    RpcInflationReward,
    RpcKeyedAccount,
    RpcKeyedAccountJsonParsed,
    RpcLogsResponse,
    RpcPerfSample,
    RpcResponseContext,
    RpcSignatureResponse,
    RpcSimulateTransactionResult,
    RpcSnapshotSlotInfo,
    RpcSupply,
    RpcTokenAccountBalance,
    RpcVersionInfo,
    RpcVote,
    RpcVoteAccountInfo,
    RpcVoteAccountStatus,
    SendTransactionResp,
    SignatureNotification,
    SignatureNotificationResult,
    SimulateTransactionResp,
    SlotInfo,
    SlotNotification,
    SlotTransactionStats,
    SlotUpdateCompleted,
    SlotUpdateCreatedBank,
    SlotUpdateDead,
    SlotUpdateFirstShredReceived,
    SlotUpdateFrozen,
    SlotUpdateNotification,
    SlotUpdateOptimisticConfirmation,
    SlotUpdateRoot,
    SubscriptionError,
    SubscriptionResult,
    UnsubscribeResult,
    ValidatorExitResp,
    VoteNotification,
    parse_notification,
    parse_websocket_message,
)
from ..solders import (
    batch_responses_from_json as _batch_from_json,
)
from ..solders import (
    batch_responses_to_json as _batch_to_json,
)
from .errors import (
    BlockCleanedUpMessage,
    BlockNotAvailableMessage,
    BlockStatusNotAvailableYetMessage,
    InternalErrorMessage,
    InvalidParamsMessage,
    InvalidRequestMessage,
    KeyExcludedFromSecondaryIndexMessage,
    LongTermStorageSlotSkippedMessage,
    MethodNotFoundMessage,
    MinContextSlotNotReachedMessage,
    NodeUnhealthyMessage,
    ParseErrorMessage,
    RpcCustomErrorFieldless,
    ScanErrorMessage,
    SendTransactionPreflightFailureMessage,
    SlotSkippedMessage,
    TransactionPrecompileVerificationFailureMessage,
    UnsupportedTransactionVersion,
    UnsupportedTransactionVersionMessage,
)

Notification = Union[
    AccountNotification,
    AccountNotificationJsonParsed,
    BlockNotification,
    LogsNotification,
    ProgramNotification,
    ProgramNotificationJsonParsed,
    SignatureNotification,
    SlotNotification,
    SlotUpdateNotification,
    RootNotification,
    VoteNotification,
]

RpcBlockUpdateError = Union[UnsupportedTransactionVersion, BlockStoreError]

WebsocketMessage = Union[
    Notification, SubscriptionResult, SubscriptionError, UnsubscribeResult
]

RPCError = Union[
    RpcCustomErrorFieldless,
    BlockCleanedUpMessage,
    SendTransactionPreflightFailureMessage,
    BlockNotAvailableMessage,
    NodeUnhealthyMessage,
    TransactionPrecompileVerificationFailureMessage,
    SlotSkippedMessage,
    LongTermStorageSlotSkippedMessage,
    KeyExcludedFromSecondaryIndexMessage,
    ScanErrorMessage,
    BlockStatusNotAvailableYetMessage,
    MinContextSlotNotReachedMessage,
    UnsupportedTransactionVersionMessage,
    ParseErrorMessage,
    InvalidRequestMessage,
    MethodNotFoundMessage,
    InvalidParamsMessage,
    InternalErrorMessage,
]

T = TypeVar("T")
Resp = Union[RPCError, T]

SlotUpdate = Union[
    SlotUpdateFirstShredReceived,
    SlotUpdateCompleted,
    SlotUpdateCreatedBank,
    SlotUpdateDead,
    SlotUpdateOptimisticConfirmation,
    SlotUpdateRoot,
    SlotUpdateFrozen,
]

RPCResult = Union[
    RPCError,
    GetAccountInfoResp,
    GetAccountInfoJsonParsedResp,
    GetAccountInfoMaybeJsonParsedResp,
    GetBalanceResp,
    GetBlockProductionResp,
    GetBlockResp,
    GetBlockCommitmentResp,
    GetBlockHeightResp,
    GetBlocksResp,
    GetBlocksWithLimitResp,
    GetBlockTimeResp,
    GetClusterNodesResp,
    GetEpochInfoResp,
    GetEpochScheduleResp,
    GetFeeForMessageResp,
    GetFirstAvailableBlockResp,
    GetGenesisHashResp,
    GetHealthResp,
    GetHighestSnapshotSlotResp,
    GetIdentityResp,
    GetInflationGovernorResp,
    GetInflationRateResp,
    GetInflationRewardResp,
    GetLargestAccountsResp,
    GetLatestBlockhashResp,
    GetLeaderScheduleResp,
    GetMaxRetransmitSlotResp,
    GetMaxShredInsertSlotResp,
    GetMinimumBalanceForRentExemptionResp,
    GetMultipleAccountsResp,
    GetMultipleAccountsJsonParsedResp,
    GetMultipleAccountsMaybeJsonParsedResp,
    GetProgramAccountsWithContextResp,
    GetProgramAccountsResp,
    GetProgramAccountsWithContextJsonParsedResp,
    GetProgramAccountsJsonParsedResp,
    GetProgramAccountsMaybeJsonParsedResp,
    GetProgramAccountsWithContextMaybeJsonParsedResp,
    GetRecentPerformanceSamplesResp,
    GetSignaturesForAddressResp,
    GetSignatureStatusesResp,
    GetSlotResp,
    GetSlotLeaderResp,
    GetSlotLeadersResp,
    GetSupplyResp,
    GetTokenAccountBalanceResp,
    GetTokenAccountsByDelegateResp,
    GetTokenAccountsByDelegateJsonParsedResp,
    GetTokenAccountsByOwnerResp,
    GetTokenAccountsByOwnerJsonParsedResp,
    GetTokenLargestAccountsResp,
    GetTokenSupplyResp,
    GetTransactionResp,
    GetTransactionCountResp,
    GetVersionResp,
    RpcVersionInfo,
    GetVoteAccountsResp,
    IsBlockhashValidResp,
    MinimumLedgerSlotResp,
    RequestAirdropResp,
    SendTransactionResp,
    SimulateTransactionResp,
]


def batch_to_json(resps: Sequence[RPCResult]) -> str:
    """Serialize a list of response objects into a single batch response JSON.

    Args:
        resps: A list of response objects.

    Returns:
        str: The batch JSON string.

    Example:
        >>> from solders.rpc.responses import batch_to_json, GetBlockHeightResp, GetFirstAvailableBlockResp
        >>> batch_to_json([GetBlockHeightResp(1233), GetFirstAvailableBlockResp(1)])
        '[{"id":0,"jsonrpc":"2.0","result":1233},{"id":0,"jsonrpc":"2.0","result":1}]'

    """  # noqa: E501
    return _batch_to_json(resps)


def batch_from_json(raw: str, parsers: Sequence[Any]) -> List[RPCResult]:
    """Deserialize a batch request JSON string into a list of request objects.

    Args:
        raw (str): The batch JSON string.
        parsers (Sequence): The classes to parse.

    Returns:
        A list of response objects.

    Example:
        >>> from solders.rpc.responses import batch_from_json, GetBlockHeightResp, GetFirstAvailableBlockResp
        >>> raw = '[{ "jsonrpc": "2.0", "result": 1233, "id": 1 },{ "jsonrpc": "2.0", "result": 111, "id": 1 }]'
        >>> batch_from_json(raw, [GetBlockHeightResp, GetFirstAvailableBlockResp])
        [GetBlockHeightResp(
            1233,
        ), GetFirstAvailableBlockResp(
            111,
        )]

    """  # noqa: E501
    return _batch_from_json(raw, parsers)


__all__ = [
    "RpcResponseContext",
    "GetAccountInfoResp",
    "GetAccountInfoJsonParsedResp",
    "GetAccountInfoMaybeJsonParsedResp",
    "GetBalanceResp",
    "RpcBlockCommitment",
    "GetBlockCommitmentResp",
    "GetBlockHeightResp",
    "RpcBlockProductionRange",
    "RpcBlockProduction",
    "GetBlockProductionResp",
    "GetBlockResp",
    "GetBlocksResp",
    "GetBlocksWithLimitResp",
    "GetBlockTimeResp",
    "RpcContactInfo",
    "GetClusterNodesResp",
    "GetEpochInfoResp",
    "GetEpochScheduleResp",
    "GetFeeForMessageResp",
    "GetFirstAvailableBlockResp",
    "GetGenesisHashResp",
    "GetHealthResp",
    "RpcSimulateTransactionResult",
    "RpcSnapshotSlotInfo",
    "GetHighestSnapshotSlotResp",
    "RpcIdentity",
    "GetIdentityResp",
    "RpcInflationGovernor",
    "GetInflationGovernorResp",
    "RpcInflationRate",
    "GetInflationRateResp",
    "RpcInflationReward",
    "GetInflationRewardResp",
    "RpcAccountBalance",
    "GetLargestAccountsResp",
    "RpcBlockhash",
    "GetLatestBlockhashResp",
    "GetLeaderScheduleResp",
    "GetMaxRetransmitSlotResp",
    "GetMaxShredInsertSlotResp",
    "GetMinimumBalanceForRentExemptionResp",
    "GetMultipleAccountsResp",
    "GetMultipleAccountsJsonParsedResp",
    "GetMultipleAccountsMaybeJsonParsedResp",
    "RpcKeyedAccount",
    "RpcKeyedAccountJsonParsed",
    "GetProgramAccountsWithContextResp",
    "GetProgramAccountsWithContextJsonParsedResp",
    "GetProgramAccountsResp",
    "GetProgramAccountsJsonParsedResp",
    "GetProgramAccountsWithContextMaybeJsonParsedResp",
    "GetProgramAccountsMaybeJsonParsedResp",
    "RpcPerfSample",
    "GetRecentPerformanceSamplesResp",
    "RpcConfirmedTransactionStatusWithSignature",
    "GetSignaturesForAddressResp",
    "GetSignatureStatusesResp",
    "GetSlotResp",
    "GetSlotLeaderResp",
    "GetSlotLeadersResp",
    "RpcSupply",
    "GetSupplyResp",
    "GetTokenAccountBalanceResp",
    "GetTokenAccountsByDelegateResp",
    "GetTokenAccountsByDelegateJsonParsedResp",
    "GetTokenAccountsByOwnerResp",
    "GetTokenAccountsByOwnerJsonParsedResp",
    "RpcTokenAccountBalance",
    "GetTokenLargestAccountsResp",
    "GetTokenSupplyResp",
    "GetTransactionResp",
    "GetTransactionCountResp",
    "RpcVersionInfo",
    "GetVersionResp",
    "RpcVoteAccountInfo",
    "RpcVoteAccountStatus",
    "GetVoteAccountsResp",
    "IsBlockhashValidResp",
    "MinimumLedgerSlotResp",
    "RequestAirdropResp",
    "ValidatorExitResp",
    "SendTransactionResp",
    "SimulateTransactionResp",
    "RpcLogsResponse",
    "RpcVote",
    "SlotTransactionStats",
    "SlotUpdateFirstShredReceived",
    "SlotUpdateCompleted",
    "SlotUpdateCreatedBank",
    "SlotUpdateDead",
    "SlotUpdateOptimisticConfirmation",
    "SlotUpdateRoot",
    "SlotUpdateFrozen",
    "AccountNotificationResult",
    "AccountNotification",
    "AccountNotificationJsonParsedResult",
    "AccountNotificationJsonParsed",
    "BlockNotificationResult",
    "BlockNotification",
    "LogsNotificationResult",
    "LogsNotification",
    "ProgramNotificationResult",
    "ProgramNotification",
    "ProgramNotificationJsonParsedResult",
    "ProgramNotificationJsonParsed",
    "RpcSignatureResponse",
    "SignatureNotificationResult",
    "SignatureNotification",
    "SlotInfo",
    "SlotNotification",
    "SlotUpdateNotification",
    "RootNotification",
    "VoteNotification",
    "SubscriptionResult",
    "SubscriptionError",
    "BlockStoreError",
    "RpcBlockUpdate",
    "UnsubscribeResult",
    "batch_to_json",
    "batch_from_json",
    "parse_notification",
    "parse_websocket_message",
    "Notification",
    "RpcBlockUpdateError",
    "WebsocketMessage",
    "RPCError",
    "Resp",
    "SlotUpdate",
    "RPCResult",
]



from typing import Union

from .solders import (
    Keypair,
    Legacy,
    NullSigner,
    Presigner,
    SanitizeError,
    Transaction,
    TransactionError,
    VersionedTransaction,
)

Signer = Union[Keypair, Presigner, NullSigner]
TransactionVersion = Union[Legacy, int]

__all__ = [
    "Legacy",
    "Transaction",
    "VersionedTransaction",
    "SanitizeError",
    "TransactionError",
    "Signer",
    "TransactionVersion",
]



from typing import Union

from .solders import (
    FailedTransactionMetadata,
    InnerInstruction,
    SimulatedTransactionInfo,
    TransactionMetadata,
)

SimulateResult = Union[SimulatedTransactionInfo, FailedTransactionMetadata]
TransactionResult = Union[TransactionMetadata, FailedTransactionMetadata]

__all__ = [
    "InnerInstruction",
    "TransactionMetadata",
    "FailedTransactionMetadata",
    "SimulatedTransactionInfo",
    "SimulateResult",
    "TransactionResult",
]



from ..solders import (
    RpcAccountInfoConfig,
    RpcBlockConfig,
    RpcBlockProductionConfig,
    RpcBlockProductionConfigRange,
    RpcBlockSubscribeConfig,
    RpcBlockSubscribeFilter,
    RpcBlockSubscribeFilterMentions,
    RpcContextConfig,
    RpcEpochConfig,
    RpcGetVoteAccountsConfig,
    RpcLargestAccountsConfig,
    RpcLargestAccountsFilter,
    RpcLeaderScheduleConfig,
    RpcProgramAccountsConfig,
    RpcRequestAirdropConfig,
    RpcSendTransactionConfig,
    RpcSignaturesForAddressConfig,
    RpcSignatureStatusConfig,
    RpcSignatureSubscribeConfig,
    RpcSimulateTransactionAccountsConfig,
    RpcSimulateTransactionConfig,
    RpcSupplyConfig,
    RpcTokenAccountsFilterMint,
    RpcTokenAccountsFilterProgramId,
    RpcTransactionConfig,
    RpcTransactionLogsConfig,
    RpcTransactionLogsFilter,
    RpcTransactionLogsFilterMentions,
)

__all__ = [
    "RpcSignatureStatusConfig",
    "RpcSendTransactionConfig",
    "RpcSimulateTransactionAccountsConfig",
    "RpcSimulateTransactionConfig",
    "RpcRequestAirdropConfig",
    "RpcLeaderScheduleConfig",
    "RpcBlockProductionConfigRange",
    "RpcBlockProductionConfig",
    "RpcGetVoteAccountsConfig",
    "RpcLargestAccountsFilter",
    "RpcLargestAccountsConfig",
    "RpcSupplyConfig",
    "RpcEpochConfig",
    "RpcAccountInfoConfig",
    "RpcProgramAccountsConfig",
    "RpcTransactionLogsFilter",
    "RpcTransactionLogsFilterMentions",
    "RpcTransactionLogsConfig",
    "RpcTokenAccountsFilterMint",
    "RpcTokenAccountsFilterProgramId",
    "RpcSignatureSubscribeConfig",
    "RpcBlockSubscribeFilter",
    "RpcBlockSubscribeFilterMentions",
    "RpcBlockSubscribeConfig",
    "RpcSignaturesForAddressConfig",
    "RpcBlockConfig",
    "RpcTransactionConfig",
    "RpcContextConfig",
]



from typing import Final

from . import solders as sd
from .solders import Clock

DEFAULT_DEV_SLOTS_PER_EPOCH: Final[int] = sd.DEFAULT_DEV_SLOTS_PER_EPOCH
""""""
DEFAULT_HASHES_PER_SECOND: Final[int] = sd.DEFAULT_HASHES_PER_SECOND
""""""
DEFAULT_HASHES_PER_TICK: Final[int] = sd.DEFAULT_HASHES_PER_TICK
""""""
DEFAULT_MS_PER_SLOT: Final[int] = sd.DEFAULT_MS_PER_SLOT
"""The expected duration of a slot in milliseconds."""
DEFAULT_SLOTS_PER_EPOCH: Final[int] = sd.DEFAULT_SLOTS_PER_EPOCH
"""The number of slots per epoch after initial network warmup."""
DEFAULT_S_PER_SLOT: Final[float] = sd.DEFAULT_S_PER_SLOT
""""""
DEFAULT_TICKS_PER_SECOND: Final[int] = sd.DEFAULT_TICKS_PER_SECOND
"""The default tick rate that the cluster attempts to achieve."""
DEFAULT_TICKS_PER_SLOT: Final[int] = sd.DEFAULT_TICKS_PER_SLOT
""""""
FORWARD_TRANSACTIONS_TO_LEADER_AT_SLOT_OFFSET: Final[
    int
] = sd.FORWARD_TRANSACTIONS_TO_LEADER_AT_SLOT_OFFSET
"""Transaction forwarding, which leader to forward to"""
GENESIS_EPOCH: Final[int] = sd.GENESIS_EPOCH
""""""
HOLD_TRANSACTIONS_SLOT_OFFSET: Final[int] = sd.HOLD_TRANSACTIONS_SLOT_OFFSET
"""Transaction forwarding, how long to hold"""
INITIAL_RENT_EPOCH: Final[int] = sd.INITIAL_RENT_EPOCH
""""""
MAX_HASH_AGE_IN_SECONDS: Final[int] = sd.MAX_HASH_AGE_IN_SECONDS
"""The window of recent blockhashes over which the bank will track signatures."""
MAX_PROCESSING_AGE: Final[int] = sd.MAX_PROCESSING_AGE
"""The maximum age of a blockhash that will be accepted by the leader"""
MAX_RECENT_BLOCKHASHES: Final[int] = sd.MAX_RECENT_BLOCKHASHES
"""Max number of recent blockhashes (one blockhash per non-skipped slot)"""
MAX_TRANSACTION_FORWARDING_DELAY: Final[int] = sd.MAX_TRANSACTION_FORWARDING_DELAY
""""""
MAX_TRANSACTION_FORWARDING_DELAY_GPU: Final[
    int
] = sd.MAX_TRANSACTION_FORWARDING_DELAY_GPU
""""""
MS_PER_TICK: Final[int] = sd.MS_PER_TICK
"""The number of milliseconds per tick."""
NUM_CONSECUTIVE_LEADER_SLOTS: Final[int] = sd.NUM_CONSECUTIVE_LEADER_SLOTS
""""""
SECONDS_PER_DAY: Final[int] = sd.SECONDS_PER_DAY
""""""
TICKS_PER_DAY: Final[int] = sd.TICKS_PER_DAY
""""""

__all__ = [
    "Clock",
    "DEFAULT_DEV_SLOTS_PER_EPOCH",
    "DEFAULT_HASHES_PER_SECOND",
    "DEFAULT_HASHES_PER_TICK",
    "DEFAULT_MS_PER_SLOT",
    "DEFAULT_SLOTS_PER_EPOCH",
    "DEFAULT_S_PER_SLOT",
    "DEFAULT_TICKS_PER_SECOND",
    "DEFAULT_TICKS_PER_SLOT",
    "FORWARD_TRANSACTIONS_TO_LEADER_AT_SLOT_OFFSET",
    "GENESIS_EPOCH",
    "HOLD_TRANSACTIONS_SLOT_OFFSET",
    "INITIAL_RENT_EPOCH",
    "MAX_HASH_AGE_IN_SECONDS",
    "MAX_PROCESSING_AGE",
    "MAX_RECENT_BLOCKHASHES",
    "MAX_TRANSACTION_FORWARDING_DELAY",
    "MAX_TRANSACTION_FORWARDING_DELAY_GPU",
    "MS_PER_TICK",
    "NUM_CONSECUTIVE_LEADER_SLOTS",
    "SECONDS_PER_DAY",
    "TICKS_PER_DAY",
]



from typing import Union

from .solders import (
    Message,
    MessageAddressTableLookup,
    MessageHeader,
    MessageV0,
    from_bytes_versioned,
    to_bytes_versioned,
)

VersionedMessage = Union[Message, MessageV0]

__all__ = [
    "Message",
    "MessageAddressTableLookup",
    "MessageHeader",
    "MessageV0",
    "to_bytes_versioned",
    "from_bytes_versioned",
    "VersionedMessage",
]



from typing import Final, Union

from .solders import (
    ADDRESS_LOOKUP_TABLE_ID,
    AddressLookupTable,
    AddressLookupTableAccount,
    LookupTableMeta,
    LookupTableStatusDeactivating,
    LookupTableStatusFieldless,
    Pubkey,
    SlotHashes,
    derive_lookup_table_address,
)
from .solders import (
    LOOKUP_TABLE_MAX_ADDRESSES as _LOOKUP_TABLE_MAX_ADDRESSES,
)
from .solders import (
    LOOKUP_TABLE_META_SIZE as _LOOKUP_TABLE_META_SIZE,
)

ID: Final[Pubkey] = ADDRESS_LOOKUP_TABLE_ID
"""Address lookup table program ID."""

LOOKUP_TABLE_MAX_ADDRESSES: Final[int] = _LOOKUP_TABLE_MAX_ADDRESSES
"""The maximum number of addresses that a lookup table can hold"""

LOOKUP_TABLE_META_SIZE: Final[int] = _LOOKUP_TABLE_META_SIZE
"""The serialized size of lookup table metadata"""

LookupTableStatusType = Union[LookupTableStatusFieldless, LookupTableStatusDeactivating]

__all__ = [
    "LookupTableStatusFieldless",
    "LookupTableStatusDeactivating",
    "AddressLookupTableAccount",
    "AddressLookupTable",
    "LookupTableMeta",
    "ID",
    "LOOKUP_TABLE_MAX_ADDRESSES",
    "LOOKUP_TABLE_META_SIZE",
    "LookupTableStatusType",
    "SlotHashes",
    "derive_lookup_table_address",
]



from typing import Sequence, Union

from ..solders import (
    AccountSubscribe,
    AccountUnsubscribe,
    BlockSubscribe,
    BlockUnsubscribe,
    GetAccountInfo,
    GetBalance,
    GetBlock,
    GetBlockCommitment,
    GetBlockHeight,
    GetBlockProduction,
    GetBlocks,
    GetBlocksWithLimit,
    GetBlockTime,
    GetClusterNodes,
    GetEpochInfo,
    GetEpochSchedule,
    GetFeeForMessage,
    GetFirstAvailableBlock,
    GetGenesisHash,
    GetHealth,
    GetHighestSnapshotSlot,
    GetIdentity,
    GetInflationGovernor,
    GetInflationRate,
    GetInflationReward,
    GetLargestAccounts,
    GetLatestBlockhash,
    GetLeaderSchedule,
    GetMaxRetransmitSlot,
    GetMaxShredInsertSlot,
    GetMinimumBalanceForRentExemption,
    GetMultipleAccounts,
    GetProgramAccounts,
    GetRecentPerformanceSamples,
    GetSignaturesForAddress,
    GetSignatureStatuses,
    GetSlot,
    GetSlotLeader,
    GetSlotLeaders,
    GetStakeActivation,
    GetSupply,
    GetTokenAccountBalance,
    GetTokenAccountsByDelegate,
    GetTokenAccountsByOwner,
    GetTokenLargestAccounts,
    GetTokenSupply,
    GetTransaction,
    GetTransactionCount,
    GetVersion,
    GetVoteAccounts,
    IsBlockhashValid,
    LogsSubscribe,
    LogsUnsubscribe,
    MinimumLedgerSlot,
    ProgramSubscribe,
    ProgramUnsubscribe,
    RequestAirdrop,
    RootSubscribe,
    RootUnsubscribe,
    SendLegacyTransaction,
    SendRawTransaction,
    SendVersionedTransaction,
    SignatureSubscribe,
    SignatureUnsubscribe,
    SimulateLegacyTransaction,
    SimulateVersionedTransaction,
    SlotSubscribe,
    SlotsUpdatesSubscribe,
    SlotsUpdatesUnsubscribe,
    SlotUnsubscribe,
    ValidatorExit,
    VoteSubscribe,
    VoteUnsubscribe,
)
from ..solders import (
    batch_requests_to_json as _batch_to_json,
)

Body = Union[
    GetAccountInfo,
    GetBalance,
    GetBlock,
    GetBlockHeight,
    GetBlockProduction,
    GetBlockCommitment,
    GetBlocks,
    GetBlocksWithLimit,
    GetBlockTime,
    GetClusterNodes,
    GetEpochInfo,
    GetEpochSchedule,
    GetFeeForMessage,
    GetFirstAvailableBlock,
    GetGenesisHash,
    GetHealth,
    GetHighestSnapshotSlot,
    GetIdentity,
    GetInflationGovernor,
    GetInflationRate,
    GetInflationReward,
    GetLargestAccounts,
    GetLatestBlockhash,
    GetLeaderSchedule,
    GetMaxRetransmitSlot,
    GetMaxShredInsertSlot,
    GetMinimumBalanceForRentExemption,
    GetMultipleAccounts,
    GetProgramAccounts,
    GetRecentPerformanceSamples,
    GetSignaturesForAddress,
    GetSignatureStatuses,
    GetSlot,
    GetSlotLeader,
    GetSlotLeaders,
    GetStakeActivation,
    GetSupply,
    GetTokenAccountBalance,
    GetTokenAccountsByDelegate,
    GetTokenAccountsByOwner,
    GetTokenLargestAccounts,
    GetTokenSupply,
    GetTransaction,
    GetTransactionCount,
    GetVersion,
    GetVoteAccounts,
    IsBlockhashValid,
    MinimumLedgerSlot,
    RequestAirdrop,
    SendLegacyTransaction,
    SendRawTransaction,
    SendVersionedTransaction,
    ValidatorExit,
    AccountSubscribe,
    BlockSubscribe,
    LogsSubscribe,
    ProgramSubscribe,
    SignatureSubscribe,
    SlotSubscribe,
    SlotsUpdatesSubscribe,
    RootSubscribe,
    VoteSubscribe,
    AccountUnsubscribe,
    BlockUnsubscribe,
    LogsUnsubscribe,
    ProgramUnsubscribe,
    SignatureUnsubscribe,
    SimulateLegacyTransaction,
    SimulateVersionedTransaction,
    SlotUnsubscribe,
    SlotsUpdatesUnsubscribe,
    RootUnsubscribe,
    VoteUnsubscribe,
]


def batch_to_json(reqs: Sequence[Body]) -> str:
    """Serialize a list of request objects into a single batch request JSON.

    Args:
        reqs: A list of request objects.

    Returns:
        str: The batch JSON string.

    Example:
        >>> from solders.rpc.requests import batch_to_json, GetClusterNodes, GetEpochSchedule
        >>> batch_to_json([GetClusterNodes(0), GetEpochSchedule(1)])
        '[{"method":"getClusterNodes","jsonrpc":"2.0","id":0},{"method":"getEpochSchedule","jsonrpc":"2.0","id":1}]'
    """  # noqa: E501
    return _batch_to_json(reqs)


__all__ = [
    "Body",
    "GetAccountInfo",
    "GetBalance",
    "GetBlock",
    "GetBlockHeight",
    "GetBlockProduction",
    "GetBlockCommitment",
    "GetBlocks",
    "GetBlocksWithLimit",
    "GetBlockTime",
    "GetEpochInfo",
    "GetFeeForMessage",
    "GetIdentity",
    "GetInflationGovernor",
    "GetInflationReward",
    "GetLargestAccounts",
    "GetLatestBlockhash",
    "GetLeaderSchedule",
    "GetMinimumBalanceForRentExemption",
    "GetMultipleAccounts",
    "GetProgramAccounts",
    "GetRecentPerformanceSamples",
    "GetSignaturesForAddress",
    "GetSignatureStatuses",
    "GetSlot",
    "GetSlotLeader",
    "GetSlotLeaders",
    "GetStakeActivation",
    "GetSupply",
    "GetTokenAccountBalance",
    "GetTokenAccountsByDelegate",
    "GetTokenAccountsByOwner",
    "GetTokenLargestAccounts",
    "GetTokenSupply",
    "GetTransaction",
    "GetTransactionCount",
    "GetVoteAccounts",
    "IsBlockhashValid",
    "RequestAirdrop",
    "SendLegacyTransaction",
    "SendRawTransaction",
    "SendVersionedTransaction",
    "SimulateLegacyTransaction",
    "SimulateVersionedTransaction",
    "ValidatorExit",
    "AccountSubscribe",
    "BlockSubscribe",
    "LogsSubscribe",
    "ProgramSubscribe",
    "SignatureSubscribe",
    "GetClusterNodes",
    "GetEpochSchedule",
    "GetFirstAvailableBlock",
    "GetGenesisHash",
    "GetHealth",
    "GetHighestSnapshotSlot",
    "GetInflationRate",
    "GetMaxRetransmitSlot",
    "GetMaxShredInsertSlot",
    "GetVersion",
    "MinimumLedgerSlot",
    "SlotSubscribe",
    "SlotsUpdatesSubscribe",
    "RootSubscribe",
    "VoteSubscribe",
    "AccountUnsubscribe",
    "BlockUnsubscribe",
    "LogsUnsubscribe",
    "ProgramUnsubscribe",
    "SignatureUnsubscribe",
    "SlotUnsubscribe",
    "SlotsUpdatesUnsubscribe",
    "RootUnsubscribe",
    "VoteUnsubscribe",
]



from .solders import (
    ACCOUNT_STORAGE_OVERHEAD,
    DEFAULT_BURN_PERCENT,
    DEFAULT_EXEMPTION_THRESHOLD,
    DEFAULT_LAMPORTS_PER_BYTE_YEAR,
    Rent,
)

__all__ = [
    "ACCOUNT_STORAGE_OVERHEAD",
    "DEFAULT_BURN_PERCENT",
    "DEFAULT_EXEMPTION_THRESHOLD",
    "DEFAULT_LAMPORTS_PER_BYTE_YEAR",
    "Rent",
]



from typing import Union

from .solders import (
    EncodedConfirmedTransactionWithStatusMeta,
    EncodedTransactionWithStatusMeta,
    InstructionErrorBorshIO,
    InstructionErrorCustom,
    InstructionErrorFieldless,
    ParsedAccountSource,
    ParsedInstruction,
    Reward,
    RewardType,
    TransactionBinaryEncoding,
    TransactionConfirmationStatus,
    TransactionDetails,
    TransactionErrorDuplicateInstruction,
    TransactionErrorFieldless,
    TransactionErrorInstructionError,
    TransactionErrorInsufficientFundsForRent,
    TransactionErrorProgramExecutionTemporarilyRestricted,
    TransactionReturnData,
    TransactionStatus,
    UiAccountsList,
    UiAddressTableLookup,
    UiCompiledInstruction,
    UiConfirmedBlock,
    UiInnerInstructions,
    UiLoadedAddresses,
    UiParsedMessage,
    UiPartiallyDecodedInstruction,
    UiRawMessage,
    UiTransaction,
    UiTransactionEncoding,
    UiTransactionStatusMeta,
    UiTransactionTokenBalance,
    VersionedTransaction,
)
from .solders import (
    ParsedAccountTxStatus as ParsedAccount,
)

UiParsedInstruction = Union[ParsedInstruction, UiPartiallyDecodedInstruction]
UiInstruction = Union[UiParsedInstruction, UiCompiledInstruction]
UiMessage = Union[UiParsedMessage, UiRawMessage]
EncodedVersionedTransaction = Union[VersionedTransaction, UiTransaction, UiAccountsList]
InstructionErrorType = Union[
    InstructionErrorFieldless,
    InstructionErrorCustom,
    InstructionErrorBorshIO,
]
TransactionErrorType = Union[
    TransactionErrorFieldless,
    TransactionErrorInstructionError,
    TransactionErrorDuplicateInstruction,
    TransactionErrorInsufficientFundsForRent,
    TransactionErrorProgramExecutionTemporarilyRestricted,
]

__all__ = [
    "UiTransactionEncoding",
    "TransactionDetails",
    "TransactionBinaryEncoding",
    "ParsedInstruction",
    "UiPartiallyDecodedInstruction",
    "UiCompiledInstruction",
    "UiAddressTableLookup",
    "UiParsedMessage",
    "UiRawMessage",
    "ParsedAccountSource",
    "ParsedAccount",
    "UiTransaction",
    "UiAccountsList",
    "UiInnerInstructions",
    "UiLoadedAddresses",
    "UiTransactionTokenBalance",
    "Reward",
    "RewardType",
    "TransactionReturnData",
    "UiTransactionStatusMeta",
    "EncodedTransactionWithStatusMeta",
    "TransactionConfirmationStatus",
    "TransactionStatus",
    "EncodedConfirmedTransactionWithStatusMeta",
    "UiConfirmedBlock",
    "InstructionErrorFieldless",
    "InstructionErrorCustom",
    "InstructionErrorBorshIO",
    "TransactionErrorFieldless",
    "TransactionErrorInstructionError",
    "TransactionErrorDuplicateInstruction",
    "TransactionErrorInsufficientFundsForRent",
    "TransactionErrorProgramExecutionTemporarilyRestricted",
    "UiParsedInstruction",
    "UiInstruction",
    "UiMessage",
    "EncodedVersionedTransaction",
    "InstructionErrorType",
    "TransactionErrorType",
]



"""The Solana LiteSVM library."""
from pathlib import Path
from typing import List, Optional, Sequence, Tuple, Union

from .solders import (
    Account,
    Clock,
    ComputeBudget,
    EpochRewards,
    EpochSchedule,
    FeatureSet,
    Hash,
    Pubkey,
    Rent,
    Signature,
    SlotHistory,
    StakeHistory,
    Transaction,
    VersionedTransaction,
)
from .solders import LiteSVM as _LiteSVM
from .transaction_metadata import SimulateResult, TransactionResult


class LiteSVM:
    """The main class in the litesvm library.

    Use this to send transactions, query accounts and configure the runtime.
    """

    def __init__(self) -> None:
        """Create a new LiteSVM instance with standard functionality enabled."""
        inner = _LiteSVM()
        self._inner = inner

    @staticmethod
    def default() -> "LiteSVM":
        """Create a new LiteSVM instance with minimal functionality enabled."""
        svm = LiteSVM()
        inner = _LiteSVM.default()
        svm._inner = inner
        return svm

    def with_compute_budget(self, budget: ComputeBudget) -> "LiteSVM":
        """Set the compute budget.

        Args:
            budget: The new compute budget

        Returns:
            The modified LiteSVM instance
        """
        self._inner.set_compute_budget(budget)
        return self

    def with_sigverify(self, sigverify: bool) -> "LiteSVM":
        """Enable or disable sigverify.

        Args:
            sigverify: if false, transaction signatures will not be checked.

        Returns:
            The modified LiteSVM instance
        """
        self._inner.set_sigverify(sigverify)
        return self

    def with_blockhash_check(self, check: bool) -> "LiteSVM":
        """Enables or disables transaction blockhash checking.

        Args:
            check: If false, the blockhash check will be skipped

        Returns:
            The modified LiteSVM instance
        """
        self._inner.set_blockhash_check(check)
        return self

    def with_sysvars(self) -> "LiteSVM":
        """Sets up the standard sysvars.

        Returns:
            The modified LiteSVM instance
        """
        self._inner.set_sysvars()
        return self

    def with_builtins(self, feature_set: Optional[FeatureSet] = None) -> "LiteSVM":
        """Adds the standard builtin programs.

        Args:
            feature_set: if provided, decides what builtins to add based on what
                features are active

        Returns:
            The modified LiteSVM instance
        """
        self._inner.set_builtins(feature_set)
        return self

    def with_lamports(self, lamports: int) -> "LiteSVM":
        """Changes the initial lamports in LiteSVM's airdrop account.

        Args:
            lamports: The number of lamports to set in the airdrop account

        Returns:
            The modified LiteSVM instance
        """
        self._inner.set_lamports(lamports)
        return self

    def with_spl_programs(self) -> "LiteSVM":
        """Adds the standard SPL programs.

        Returns:
            The modified LiteSVM instance
        """
        self._inner.set_spl_programs()
        return self

    def with_transaction_history(self, capacity: int) -> "LiteSVM":
        """Changes the capacity of the transaction history.

        Args:
            capacity: How many transactions to store in history.
                Set this to 0 to disable transaction history
                and allow duplicate transactions.

        Returns:
            The modified LiteSVM instance
        """
        self._inner.set_transaction_history(capacity)
        return self

    def with_log_bytes_limit(self, limit: Optional[int] = None) -> "LiteSVM":
        """Set a limit for transaction logs, beyond which they will be truncated.

        Args:
            limit: The limit in bytes. If None, no limit is enforced.

        Returns:
            The modified LiteSVM instance
        """
        self._inner.set_log_bytes_limit(limit)
        return self

    def with_precompiles(self, feature_set: Optional[FeatureSet] = None) -> "LiteSVM":
        """Adds the standard precompiles.

        Args:
            feature_set: if provided, decides what precompiles to add based on what
                features are active

        Returns:
            The modified LiteSVM instance
        """
        self._inner.set_precompiles(feature_set)
        return self

    def minimum_balance_for_rent_exemption(self, data_len: int) -> int:
        """Calculates the minimum balance required to make an account rent exempt.

        Args:
            data_len: The number of bytes in the account.

        Returns:
            The required balance in lamports
        """
        return self._inner.minimum_balance_for_rent_exemption(data_len)

    def get_account(self, address: Pubkey) -> Optional[Account]:
        """Return the account at the given address.

        If the account is not found, None is returned.

        Args:
            address: The account address to look up.

        Returns:
            The account object, if the account exists.
        """
        return self._inner.get_account(address)

    def set_account(self, address: Pubkey, account: Account) -> None:
        """Create or overwrite an account, subverting normal runtime checks.

        This method exists to make it easier to set up artificial situations
        that would be difficult to replicate by sending individual transactions.
        Beware that it can be used to create states that would not be reachable
        by sending transactions!

        Args:
            address: The address to write to.
            account: The account object to write.
        """
        self._inner.set_account(address, account)

    def get_balance(self, address: Pubkey) -> Optional[int]:
        """Gets the balance of the provided account address.

        Args:
            address: The account address.

        Returns:
            The account's balance in lamports.
        """
        return self._inner.get_balance(address)

    def latest_blockhash(self) -> Hash:
        """Gets the latest blockhash.

        Since LiteSVM doesn't have blocks, this is an arbitrary
        value controlled by LiteSVM.

        Returns:
            The designated latest blockhash.
        """
        return self._inner.latest_blockhash()

    def get_transaction(self, signature: Signature) -> Optional[TransactionResult]:
        """Gets a transaction from the transaction history.

        Args:
            signature: The transaction signature

        Returns:
            The transaction, if it is found in the history.
        """
        return self._inner.get_transaction(signature)

    def airdrop(self, address: Pubkey, lamports: int) -> TransactionResult:
        """Airdrops the lamport amount specified to the given address.

        Args:
            address: The airdrop recipient.
            lamports: The amount to airdrop.

        Returns:
            The transaction result.
        """
        return self._inner.airdrop(address, lamports)

    def add_program_from_file(self, program_id: Pubkey, path: Path) -> None:
        """Adds an SBF program to the test environment from the file specified.

        Args:
            program_id: The program ID.
            path: The path to the .so file.
        """
        return self._inner.add_program_from_file(program_id, path)

    def add_program(self, program_id: Pubkey, program_bytes: bytes) -> None:
        """Adds an SBF program to the test environment.

        Args:
            program_id: The program ID.
            program_bytes: The raw bytes of the compiled program.
        """
        return self._inner.add_program(program_id, program_bytes)

    def send_transaction(
        self, tx: Union[Transaction, VersionedTransaction]
    ) -> TransactionResult:
        """Processes a transaction and returns the result.

        Args:
            tx: The transaction to send.

        Returns:
            TransactionMetadata if transaction succeeds, else FailedTransactionMetadata
        """
        return self._inner.send_transaction(tx)

    def simulate_transaction(
        self, tx: Union[Transaction, VersionedTransaction]
    ) -> SimulateResult:
        """Simulates a transaction.

        Args:
            tx: The transaction to simulate

        Returns:
            SimulatedTransactionInfo if sim succeeds, else FailedTransactionMetadata
        """
        return self._inner.simulate_transaction(tx)

    def expire_blockhash(self) -> None:
        """Expires the current blockhash.

        The return value of `latest_blockhash()` will be different after calling this.
        """
        self._inner.expire_blockhash()

    def warp_to_slot(self, slot: int) -> None:
        """Warps the clock to the specified slot.

        This is a convenience wrapper around `set_clock()`.

        Args:
            slot: The new slot.
        """
        self._inner.warp_to_slot(slot)

    def get_clock(self) -> Clock:
        """Get the cluster clock.

        Returns:
            the clock object.
        """
        return self._inner.get_clock()

    def set_clock(self, clock: Clock) -> None:
        """Overwrite the clock sysvar.

        Args:
            clock: The clock object.
        """
        self._inner.set_clock(clock)

    def get_epoch_rewards(self) -> EpochRewards:
        """Get the EpochRewards sysvar.

        Returns:
            the EpochRewards object.
        """
        return self._inner.get_epoch_rewards()

    def set_epoch_rewards(self, rewards: EpochRewards) -> None:
        """Overwrite the EpochRewards sysvar.

        Args:
            rewards: The EpochRewards object.
        """
        self._inner.set_epoch_rewards(rewards)

    def get_epoch_schedule(self) -> EpochSchedule:
        """Get the EpochSchedule sysvar.

        Returns:
            the EpochSchedule object.
        """
        return self._inner.get_epoch_schedule()

    def set_epoch_schedule(self, schedule: EpochSchedule) -> None:
        """Overwrite the EpochSchedule sysvar.

        Args:
            schedule: The EpochSchedule object.
        """
        self._inner.set_epoch_schedule(schedule)

    def get_last_restart_slot(self) -> int:
        """Get the last restart slot sysvar.

        Returns:
            the last restart slot.
        """
        return self._inner.get_last_restart_slot()

    def set_last_restart_slot(self, slot: int) -> None:
        """Overwrite the last restart slot sysvar.

        Args:
            slot: The last restart slot.
        """
        self._inner.set_last_restart_slot(slot)

    def get_rent(self) -> Rent:
        """Get the cluster rent.

        Returns:
            The rent object.
        """
        return self._inner.get_rent()

    def set_rent(self, rent: Rent) -> None:
        """Overwrite the rent sysvar.

        Args:
            rent: The new rent object.
        """
        self._inner.set_rent(rent)

    def get_slot_hashes(self) -> List[Tuple[int, Hash]]:
        """Get the SlotHashes sysvar.

        Returns:
            The SlotHash array.
        """
        return self._inner.get_slot_hashes()

    def set_slot_hashes(self, hashes: Sequence[Tuple[int, Hash]]) -> None:
        """Overwrite the SlotHashes sysvar.

        Args:
            hashes: The SlotHash array.
        """
        self._inner.set_slot_hashes(hashes)

    def get_slot_history(self) -> SlotHistory:
        """Get the SlotHistory sysvar.

        Returns:
            The SlotHistory object.
        """
        return self._inner.get_slot_history()

    def set_slot_history(self, history: SlotHistory) -> None:
        """Overwrite the SlotHistory sysvar.

        Args:
            history: The SlotHistory object
        """
        self._inner.set_slot_history(history)

    def get_stake_history(self) -> StakeHistory:
        """Get the StakeHistory sysvar.

        Returns:
            The StakeHistory object.
        """
        return self._inner.get_stake_history()

    def set_stake_history(self, history: StakeHistory) -> None:
        """Overwrite the StakeHistory sysvar.

        Args:
            history: The StakeHistory object
        """
        self._inner.set_stake_history(history)


__all__ = ["FeatureSet", "LiteSVM"]



from typing import Union

from ..solders import Memcmp, RpcFilterTypeFieldless

RpcFilterType = Union[int, Memcmp, RpcFilterTypeFieldless]

__all__ = [
    "Memcmp",
    "RpcFilterTypeFieldless",
    "RpcFilterType",
]
